/*
    Copyright (c) 2020 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/
#include "notifications.h"

#include <ds-api/event-mock.h>
#include <ds-net/session-base.h>
#include <ds-net/signalr-mock.h>
#include <ds/asio/test.h>
#include <ds/catch/catch.h>

using namespace trompeloeil;
using namespace std::literals;

namespace {

static const char* TAGS = "[dsApiNotifications][dsApi]";

SCENARIO("dsApiNotifications", TAGS) {
  auto clockMocker = ds::ClockMocker{};
  auto expectClose = std::unique_ptr<trompeloeil::expectation>{}; // close can come from io handler destructor
  auto loop = ds::asio::TestLoopResource{};
  auto signalrAcceptorPtr = std::make_unique<dsNet::SignalrAcceptorMock>();
  auto& signalrAcceptor = *signalrAcceptorPtr;
  auto eventHub = dsApi::EventHubMock{};

  SECTION("notisignalAcceptor runs automatically") {
    auto signalrAcceptorHooks = dsNet::SignalrAcceptorHooks{};
    REQUIRE_CALL(signalrAcceptor, asyncRun(_)).LR_SIDE_EFFECT(signalrAcceptorHooks = _1);
    auto notificationsAcceptor = dsApi::makeNotifications(loop, std::move(signalrAcceptorPtr), eventHub);

    SECTION("nothing") {}
    SECTION("signalr session accept -> signalr session asyncRun") {
      auto signalrSessionHooks = dsNet::SignalrSessionHooks{};
      auto signalrSession = std::make_shared<dsNet::SignalrSessionMock>(loop, "id1");
      REQUIRE_CALL(*signalrSession, asyncRun(_)).LR_SIDE_EFFECT(signalrSessionHooks = _1);
      signalrAcceptorHooks.accept(std::shared_ptr{signalrSession});
      expectClose = NAMED_REQUIRE_CALL(*signalrSession, close());

      auto checkClose = [&] {
        signalrSessionHooks.closed(boost::asio::error::operation_aborted);
        expectClose.reset();
      };
      SECTION("nothing") {}
      SECTION("message stream closed") { checkClose(); }
      SECTION("signalr session opened -> default subscriptions are applied") {
        auto eventSink = (dsApi::EventSink*){nullptr};
        REQUIRE_CALL(eventHub, subscribe(dsApi::Subscription{.type = dsApi::EventType{"apartmentStatusChanged"},
                                                             .expiresAt = ds::Clock::time_point::max()},
                                         _))
            .LR_SIDE_EFFECT(CHECK(eventSink == &_2));
        REQUIRE_CALL(eventHub, subscribe(dsApi::Subscription{.type = dsApi::EventType{"apartmentStructureChanged"},
                                                             .expiresAt = ds::Clock::time_point::max()},
                                         _))
            .LR_SIDE_EFFECT(eventSink = &_2);
        signalrSessionHooks.opened();

        SECTION("nothing") {}
        SECTION("message stream closed") { checkClose(); }
        SECTION("signal subscribe -> eventHub subscribe") {
          REQUIRE_CALL(
              eventHub,
              subscribe(dsApi::Subscription{.type = dsApi::EventType{"typeX"}, .expiresAt = ds::Clock::now() + 10s}, _))
              .LR_SIDE_EFFECT(CHECK(eventSink == &_2));
          signalrSessionHooks.invokeNoReply("subscribe",
                                            {ds::fromJson<ds::JsonFragment>(R"({"type":"typeX","timeout":10})")});
        }
        SECTION("subscribed evenHub event -> event") {
          auto event = ds::Box{dsApi::Event{.type = dsApi::EventType{"apartmentStatusChanged"}}};
          REQUIRE_CALL(*signalrSession, invokeNoReply("event", std::vector{ds::toJsonFragment(*event)}))
              .LR_SIDE_EFFECT(loop.stop());
          eventHub.m_event.emit(event);
          loop.run();
        }
      }
    }
  }
}

} // namespace
