#include "wss-client.h"

#include <ds-net/websocket.h>
#include <ds/conf.h>
#include <ds/system.h>

namespace dsBackend {

std::unique_ptr<dsNet::MessageStreamClient> makeWSSClient(ds::asio::ResourceBase &parent,
                                                          boost::asio::ssl::context &sslContext) {
  // We could eliminate sslContext from argument.
  // But we would need an option to attach it to returned std::unique_ptr<dsBackend::StreamClient>.
  return dsNet::makeWSSClient(
      parent,
      {.url = parent.system().conf().readOr<std::string>("backendConnectorUrl",
                                                         [&] {
                                                           const auto release = parent.system().release();
                                                           switch (release) {
                                                             case ds::Release::devel:
                                                               return "wss://dsbcdev.devapi.digitalstrom.com:443/dss";
                                                             case ds::Release::testing:
                                                               return "wss://dsbcalpha.devapi.digitalstrom.com:443/dss";
                                                             case ds::Release::fieldtest:
                                                               return "wss://dsbc.api.digitalstrom.com:443/dss";
                                                             case ds::Release::production:
                                                               return "wss://dsbc.api.digitalstrom.com:443/dss";
                                                           }
                                                           DS_ASSERT("Unknown release", release);
                                                           return ""; // To work around -Werror=return-type in gcc 7.3
                                                         }())},
      sslContext);
}

} // namespace dsBackend
