#pragma once

#include <ds-apartment/api-revision-fwd.h>
#include <ds-api/event-fwd.h>
#include <ds-api/service-fwd.h>
#include <ds-backend/connector-fwd.h>
#include <ds-model/apartment-api.h>
#include <ds-model/model-fwd.h>
#include <ds/asio/resource-fwd.h>

#include <boost/noncopyable.hpp>

#include "handler/system_states.h"
#include "model/apartment.h"

///
/// Glue between dss model and ds model
///

template <>
struct DSToEnum<dsModel::ZoneApplicationType, dss::ApplicationType, void> {
  static std::optional<dsModel::ZoneApplicationType> tryTo(dss::ApplicationType x);
};

template <>
struct DSToEnum<dsModel::ApartmentApplicationType, dss::ApplicationType, void> {
  static std::optional<dsModel::ApartmentApplicationType> tryTo(dss::ApplicationType x);
};

namespace dss {
struct DecodedApplicationId {
  int zoneId{};
};

DecodedApplicationId decodeId(const dsModel::ApplicationId&);

std::vector<dsModel::ClusterId> resourceIds(const std::vector<dsModel::Cluster>&);

dsModel::Sensor makeDsModelSensor();
dsModel::Button makeDsModelButton();
dsModel::Output makeDsModelOutput();

dsModel::ZoneApplicationType zoneApplication(const int& application);
dsModel::OutputMode outputMode(const int& mode);
std::optional<int> outputLevels(const int& mode);
dsModel::ButtonMode buttonMode(const ButtonInputMode& mode);

dsModel::ButtonType buttonType(const int& type, const int& activeGroup);
dsModel::OutputType outputType(ChannelType type);

std::vector<dsModel::Sensor> getSensors(const Device& device);
std::vector<dsModel::Button> getButtons(const Device& device);
std::vector<dsModel::Output> getOutputs(const Device& device, std::vector<double>&& levelThresholds);

class DsModel : public dsModel::ApartmentApi {
public:
  virtual void handleEvent(const Event& event) = 0; // for mocking purposes
};

struct DsModelHooks {
  // to support unit tests
  bool pollersActive = true;
  std::function<void()> apartmentStructurePolled{[] {}};
  std::function<void()> apartmentStatusPolled{[] {}};
};

std::unique_ptr<DsModel> makeDsModel(ds::asio::ResourceBase& parent, dsBackend::Connector&, Apartment&,
                                     EventInterpreter&, dsApi::EventHub&, ds::asio::ResourceContext&,
                                     dsApartment::ApiRevision&, DsModelHooks&& = {});

} // namespace dss

DS_REFLECT_STRUCT((dss), DecodedApplicationId, (zoneId));
