/*
    Copyright (c) 2020 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/
#include "session-base.h"

#include <ds/vector.h>

#include <boost/range/adaptor/transformed.hpp>

namespace dsNet {

SessionsHook::SessionsHook(Sessions& sessions) {
  sessions.push_back(*this);
}

SessionsHook::~SessionsHook() = default;

Sessions::Sessions(ds::asio::ResourceBase& parent, std::string&& id) : ds::asio::Resource<>{parent, std::move(id)} {}

Sessions::~Sessions() {
  // upgrade (weak) pointers to shared_pointers
  // we cannot iterate over list where the items are being delete during iteration
  auto sharedSessions =
      ds::makeVectorNoReserve(m_sessions | boost::adaptors::transformed([](auto&& x) { return x.sharedSession(); }));
  for (auto& session : sharedSessions) {
    session->close();
  }
}

void Sessions::push_back(SessionsHook& session) {
  m_sessions.push_back(session);
}

} // namespace dsNet
