#pragma once

#include <ds-model/apartment-api-fwd.h>
#include <ds-model/model-ids.h>
#include <ds-model/scenarios-fwd.h>
#include <ds/asio/resource-fwd.h>

#include "model/apartment.h"

///
/// Glue between dss model and ds model
///

namespace dsModel {
std::string_view idView(const dsModel::Scenario&);
dsModel::ScenarioId id(const dsModel::Scenario&);
} // namespace dsModel

namespace dss {

struct ApplicationZoneScenarioId {
  int zone;
  int group;
  int scene;
};
BOOST_STRONG_TYPEDEF(std::string, UserDefinedActionId);
struct DeviceScenarioId {
  dsModel::DSDeviceId device;
  dsModel::DeviceScenarioActionId action;
};
using ScenarioId = std::variant<std::monostate, dsModel::ApplicationZoneScenarioId, dsModel::UserDefinedActionId,
                                dsModel::DeviceScenarioId>;
using DecodedScenarioId = std::variant<ApplicationZoneScenarioId, UserDefinedActionId, DeviceScenarioId>;

std::vector<ApplicationZoneScenarioId> zoneApplicationScenarioIds(dss::Zone&);
std::vector<dsModel::ScenarioId> zoneScenarioIds(dss::Zone&);

ScenarioId decodeId(const dsModel::ScenarioId& id);
DecodedScenarioId decodeDecodeId(const dsModel::ScenarioId& id);
dsModel::ScenarioId encodeId(const ScenarioId& id);

ApplicationZoneScenarioId decodeId(const dsModel::ApplicationZoneScenarioId& id);
dsModel::ApplicationZoneScenarioId encodeId(const ApplicationZoneScenarioId& id);

UserDefinedActionId decodeId(const dsModel::UserDefinedActionId& id);
dsModel::UserDefinedActionId encodeId(const UserDefinedActionId& id);

DeviceScenarioId decodeId(const dsModel::DeviceScenarioId& id);
dsModel::DeviceScenarioId encodeId(const DeviceScenarioId& id);

std::unique_ptr<dsModel::ScenariosApi> makeDsScenarios(ds::asio::ResourceBase& parent, Apartment&);

} // namespace dss

DS_REFLECT_STRUCT((dss), ApplicationZoneScenarioId, (zone)(group)(scene));
DS_REFLECT_STRUCT((dss), DeviceScenarioId, (device)(action));
