/*
    Copyright (c) 2019 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/

#include "loop.h"
#include <ds/log.h>
#include <thread>
#include "latency-watchdog.h"
#include "loop-impl.h"

using namespace std::literals;
static ds::LogChannel dsLogChannel("dsAsioLoop");

namespace ds::asio {

struct Loop::Impl {
  LatencyWatchdogConfig config;
  LoopImpl impl{};
  LatencyWatchdog latencyWatchdog{impl.io, config};
};

Loop::Loop(ds::Conf &conf, std::string &&name, LatencyWatchdogConfig &&config)
    : m_impl(new Impl{.config = conf.readOr<ds::asio::LatencyWatchdogConfig>(name.c_str(), std::move(config)),
                      .impl = {.name = std::move(name)}}),
      m_context{m_impl->impl.contextData, io(), ds::system()} {}
Loop::~Loop() = default;

boost::asio::io_service &Loop::io() {
  return m_impl->impl.io;
}

} // namespace ds::asio
