/*
 *  Copyright (c) 2019 digitalSTROM.org, Zurich, Switzerland
 *
 *  This file is part of digitalSTROM Server.
 *
 *  digitalSTROM Server is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  digitalSTROM Server is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
 */

#include "moderation-timer.h"

static ds::LogChannel dsLogChannel("dsAsioModerationTimer");

namespace ds::asio {

using namespace std::literals;

ModerationTimer::ModerationTimer(boost::asio::io_service& io, std::function<void()> fire, Timer::duration period)
    : m_timer{io}, m_fire{std::move(fire)}, m_period{period} {}

ModerationTimer::~ModerationTimer() {}

void ModerationTimer::trigger() {
  DS_DEBUG_ENTER(m_isPending);
  if (m_isPending) {
    return;
  }

  m_isPending = true;
  const auto now = Timer::clock_type::now();
  const auto fromNow = [&]() -> Timer::duration {
    if (const auto& lastFiredAt = m_lastFiredAt) {
      const auto idle = now - *lastFiredAt;
      if (idle < m_period) {
        return m_period - idle;
      } else {
        return 0s;
      }
    } else {
      return 0s;
    }
  }();
  m_timer.randomlyExpiresFromNow(fromNow, [this] { fire(); });
}

void ModerationTimer::cancel() {
  m_isPending = false;
  m_timer.cancel();
}

void ModerationTimer::fire() {
  DS_ASSERT(m_isPending);
  m_isPending = false;
  m_lastFiredAt = Timer::clock_type::now();
  m_fire();
}

} // namespace ds::asio
