/*
 *  Copyright (c) 2019 digitalSTROM.org, Zurich, Switzerland
 *
 *  This file is part of digitalSTROM Server.
 *
 *  digitalSTROM Server is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  digitalSTROM Server is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include <ds/log.h>
#include <functional>
#include <optional>
#include "timer.h"

namespace ds::asio {

using namespace std::literals;

///
/// Moderates triggers into fires so that fires don't happen too often.
///
/// First trigger fires immediately.
/// Further triggers are delayed and merged to enforce `period` between fires.
///
class ModerationTimer {
public:
  ModerationTimer(boost::asio::io_service& io, std::function<void()> fire, Timer::duration period);
  ~ModerationTimer();

  void trigger();
  void cancel();

private:
  Timer m_timer;
  const std::function<void()> m_fire;
  const Timer::duration m_period;
  bool m_isPending{};
  std::optional<Timer::time_point> m_lastFiredAt{};

  void fire();
};

} // namespace ds::asio
