/*
    Copyright (c) 2020 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/
#include "property.h"

#include <ds/asio/ds-signal-mock.h>
#include <ds/asio/test.h>
#include <ds/catch/catch.h>

#include <trompeloeil.hpp>

namespace {

static const char* TAGS = "[dsAsioProperty]";
using namespace std::literals;
using namespace trompeloeil;

DS_SLOT_MOCK1(ChangedMock, ds::Box<std::string>);

TEST_CASE("dsAsioProperty", TAGS) {
  ds::asio::TestLoopResource loop;

  auto property = ds::Property<std::string>{loop, "a"};
  auto propertyChanged = ChangedMock{loop, property.changed()};

  SECTION("get") { CHECK(property.get() == "a"); }

  SECTION("set with different value mutates state and emits signal") {
    property.set("b"s);
    CHECK(property.get() == "b");
    REQUIRE_CALL(propertyChanged, call("b"s)).LR_SIDE_EFFECT(loop.stop());
    loop.run();
  }

  SECTION("set with the same object preserves state does not emit signal") {
    property.set("a"s);
    CHECK(property.get() == "a");
  }

  // fail if slots were scheduled
  loop.postPostStopRun();
}

} // namespace
