#include "reconnect-timer.h"

using namespace std::literals;

namespace ds::asio {

ReconnectTimer::ReconnectTimer(boost::asio::io_service& io) : m_timer{io} {}

ReconnectTimer::~ReconnectTimer() {}

void ReconnectTimer::connected() {
  m_connectedAt = Timer::clock_type::now();
}

void ReconnectTimer::resetInterval() {
  m_timer.cancel();
  m_intervalsIt = m_intervals.cbegin();
}

Timer::duration ReconnectTimer::raiseInterval() {
  if (m_connectedAt) {
    if (Timer::clock_type::now() - *m_connectedAt > m_resetIntervalsAfterConnectedDuration) {
      // connection was up long enough to reset the intervals
      resetInterval();
    }
    m_connectedAt = {};
  }

  auto result = interval();
  if (!isLastInterval()) {
    ++m_intervalsIt;
  }
  return result;
}

void ReconnectTimer::raiseIntervalUpTo(Timer::duration duration) {
  while (interval() < duration && !isLastInterval()) {
    raiseInterval();
  }
}

} // namespace ds::asio
