/*
 *  Copyright (c) 2016 digitalSTROM.org, Zurich, Switzerland
 *
 *  This file is part of digitalSTROM Server.
 *
 *  digitalSTROM Server is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  digitalSTROM Server is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include <ds/clock.h>
#include <ds/json.h>
#include <ds/reflect.h>
#include <boost/asio/io_service.hpp>
#include <functional>

namespace ds::asio {

using namespace std::literals;

struct StatePollerConfig {
  ds::Clock::duration moderate{};
  ds::Clock::duration loop{};
};

struct StatePollerHooks {
  ~StatePollerHooks();
  std::function<ds::JsonFragment()> poll;

  /// Called when change is detected
  std::function<void(ds::JsonFragment&&)> fire;

  /// Called when the same value was polled.
  /// Primarily to support unit tests.
  std::function<void()> polledAndNoFire{[] {}};
};

///
/// `Poll`s json state and fires `changed` when change is detected.
///
class StatePoller {
  struct Impl;
  const std::unique_ptr<Impl> m_impl;

public:
  StatePoller(boost::asio::io_service& io, std::string&& id, StatePollerConfig&& config, StatePollerHooks&& hooks);
  ~StatePoller();

  /// Thread safe
  void setActive(bool);

  /// Poll state.
  ///
  /// Poll requests are moderated.
  /// Thread safe
  void poll();
};

} // namespace ds::asio

DS_REFLECT_STRUCT((ds)(asio), StatePollerConfig, (moderate)(loop));
