/*
    Copyright (c) 2019 digitalSTROM AG, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#include <boost/asio/io_service.hpp>
#include <chrono>
#include <functional>
#include <memory>
#include "timer.h"

namespace ds::asio {

/// Software watchdog that guards that the process is able
/// to allocate new file descriptors.
///
/// When file descriptors are leaking and process reaches the limit
/// bad things tend to happen.
/// The best recovery tends to be process restart.
class FileDescriptorsWatchdog {
public:
  FileDescriptorsWatchdog(boost::asio::io_service &io, std::chrono::milliseconds period);
  ~FileDescriptorsWatchdog();

  /// Set function called when watchog fires. Default asserts.
  void setFire(std::function<void(const std::exception &)> &&x) { m_fire = std::move(x); }

private:
  Timer m_timer;
  std::chrono::milliseconds m_period;
  std::function<void(const std::exception &)> m_fire;

  void asyncLoop();
  void defaultFire(const std::exception &e);
};

} // namespace ds::asio
