/*
    Copyright (c) 2010 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#include <chrono>
#include <iostream>

namespace ds {

/// Measures elapsed time since the class is created.
///
/// The ostream operator is handy for logging elapsed time.
///
class Elapsed {
public:
  typedef std::chrono::steady_clock Clock;
  typedef std::chrono::steady_clock::duration Duration;
  Elapsed() : m_start(Clock::now()) {}

  Duration elapsed() const { return Clock::now() - m_start; }
  void restart() { m_start = Clock::now(); }

private:
  std::chrono::steady_clock::time_point m_start;
};

// Serializes elapsed time.
std::ostream& operator<<(std::ostream& ostream, const Elapsed& x);

} // namespace ds
