/*
    Copyright (c) 2016 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/
#pragma once
#include <gsl.h>
#include <cstring>
#include <optional>
#include "common.h"

namespace ds {

template <typename T>
struct Enum {
  gsl::span<const std::pair<T, const char*>> options;
  gsl::czstring name; // for better error messages
};

template <class T, typename Enable = decltype(dsEnum(T()))>
using enum_t = T;

} // namespace

template <typename R, typename T, typename Enable = void>
struct DSToEnum;

template <typename R>
struct DSToEnumRString {
  static std::optional<R> tryTo(gsl::cstring_span x) {
    for (const auto& option : dsEnum(R{}).options) {
      if (x == option.second) {
        return option.first;
      }
    }
    return std::nullopt;
  }
};

template <typename R, typename T>
struct DSToEnum<R, T, std::void_t<ds::enum_t<R>, decltype(gsl::cstring_span(T{}))>> : public ::DSToEnumRString<R> {};

template <typename R>
struct DSToEnum<R, int, void> {
  static std::optional<R> tryTo(int x) {
    for (const auto& option : dsEnum(R{}).options) {
      if (static_cast<int>(option.first) == x) {
        return option.first;
      }
    }
    return std::nullopt;
  }
};

template <typename R, typename T>
struct DSToEnum<R, std::optional<T>, void> {
  static std::optional<R> tryTo(const std::optional<T>& x) {
    if (x) {
      return DSToEnum<R, T>::tryTo(*x);
    }
    return std::nullopt;
  }
};

template <typename R, typename T, typename Enable = void>
struct DSFromEnum;

template <typename T>
struct DSFromEnum<const char*, T, void> {
  static std::optional<const char*> tryFrom(T x) {
    for (const auto& option : dsEnum(x).options) {
      if (option.first == x) {
        return option.second;
      }
    }
    return std::nullopt;
  }
};
template <typename T>
struct DSFromEnum<std::string, T, void> {
  static std::optional<std::string> tryFrom(T x) {
    for (const auto& option : dsEnum(x).options) {
      if (option.first == x) {
        return std::string(option.second);
      }
    }
    return std::nullopt;
  }
};
template <typename T>
struct DSFromEnum<int, T, void> {
  static std::optional<int> tryFrom(T x) {
    for (const auto& option : dsEnum(x).options) {
      if (option.first == x) {
        return static_cast<int>(x);
      }
    }
    return std::nullopt;
  }
};
