/*
    Copyright (c) 2009,2010,2012 digitalSTROM.org, Zurich, Switzerland

    Authors: Patrick Staehlin, futureLAB AG <pstaehlin@futurelab.ch>
             Michael Troß, aizo GmbH <michael.tross@aizo.com>

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef EVENTINTERPRETERPLUGINS_H_
#define EVENTINTERPRETERPLUGINS_H_

#include "event.h"

#include <unordered_map>

#include <memory>

#include "webservice_api.h"

namespace dss {

  class Apartment;
  class InternalEventRelayTarget;

  class EventInterpreterPluginRaiseEvent : public EventInterpreterPlugin {
  private:
    void applyOptionsWithSuffix(std::shared_ptr<const SubscriptionOptions> _options, const std::string& _suffix, std::shared_ptr<Event> _event, bool _onlyOverride);
  public:
    EventInterpreterPluginRaiseEvent(EventInterpreter* _pInterpreter);
    virtual ~EventInterpreterPluginRaiseEvent();

    virtual void handleEvent(Event& _event, const EventSubscription& _subscription);
  }; // EventInterpreterPluginRaiseEvent

  class ScriptContextWrapper;
  class PropertyNode;
  typedef std::shared_ptr<PropertyNode> PropertyNodePtr;


  //-------------------------------------------------- Event Relay

  class EventInterpreterInternalRelay;

  class EventRelayTarget {
  public:
    EventRelayTarget(EventInterpreterInternalRelay& _relay)
    : m_Relay(_relay)
    { } // ctor
    virtual ~EventRelayTarget();

    virtual void handleEvent(Event& _event, const EventSubscription& _subscription) = 0;
    virtual void subscribeTo(std::shared_ptr<EventSubscription> _pSubscription);
    virtual void unsubscribeFrom(const std::string& _subscriptionID);
  protected:
    EventInterpreterInternalRelay& getRelay() { return m_Relay; }
  private:
    EventInterpreterInternalRelay& m_Relay;
    typedef std::vector<std::shared_ptr<EventSubscription> > SubscriptionPtrList;
    SubscriptionPtrList m_Subscriptions;
  }; // EventRelayTarget

  class EventInterpreterInternalRelay : public EventInterpreterPlugin {
  public:
    EventInterpreterInternalRelay(EventInterpreter* _pInterpreter);
    virtual ~EventInterpreterInternalRelay();

    virtual void handleEvent(Event& _event, const EventSubscription& _subscription);

    static const char* getPluginName() { return "internal_relay"; }
    EventInterpreter& getEventInterpreter() { return EventInterpreterPlugin::getEventInterpreter(); }
  protected:
    friend class EventRelayTarget;
    void registerSubscription(EventRelayTarget* _pTarget, const std::string& _subscriptionID);
    void removeSubscription(const std::string& _subscriptionID);
  private:
    std::unordered_map<std::string, EventRelayTarget*> m_IDTargetMap;
  }; // EventInterpreterInternalRelay

  class EventInterpreterPluginSendmail : public EventInterpreterPlugin {
   private:
     std::deque<std::string> m_MailFiles;
     std::string m_mailq_dir;
     static bool m_active;
   public:
     EventInterpreterPluginSendmail(EventInterpreter* _pInterpreter);
     virtual ~EventInterpreterPluginSendmail();
     virtual void handleEvent(Event& _event, const EventSubscription& _subscription);
   }; // EventInterpreterPluginSendmail

  class EventInterpreterPluginExecutionDeniedDigest : public EventInterpreterPlugin {
  private:
    std::ostringstream m_digest;
    DateTime m_timestamp;
    boost::mutex m_digest_mutex;
    bool m_check_scheduled;

  public:
    EventInterpreterPluginExecutionDeniedDigest(EventInterpreter* _pInterpreter);
    virtual void handleEvent(Event& _event, const EventSubscription& _subscription);
  };

  class EventInterpreterPluginApartmentChange : public EventInterpreterPlugin {
  public:
    EventInterpreterPluginApartmentChange(EventInterpreter* _pInterpreter);
    virtual void handleEvent(Event& _event, const EventSubscription& _subscription);
  };

  class EventInterpreterWebservicePlugin : public EventInterpreterPlugin,
                                           private PropertyListener {
  private:
    __DECL_LOG_CHANNEL__
    virtual void propertyChanged(PropertyNodePtr _caller,
                                 PropertyNodePtr _changedNode);
  public:
    static std::string ConnectionKeepAlive;
    EventInterpreterWebservicePlugin(EventInterpreter* _pInterpreter);
    virtual ~EventInterpreterWebservicePlugin();
    virtual void subscribe();
    virtual void handleEvent(Event& _event, const EventSubscription& _subscription);
  private:
    /** do not call DSS::getInstance() in destructor */
    PropertyNodePtr websvcEnabledNode;
  };

  class EventInterpreterSensorMonitorPlugin : public EventInterpreterPlugin {
    private:
      __DECL_LOG_CHANNEL__
    public:
      EventInterpreterSensorMonitorPlugin(EventInterpreter* _pInterpreter);
      virtual ~EventInterpreterSensorMonitorPlugin() {};
      virtual void subscribe();
      virtual void handleEvent(Event& _event, const EventSubscription& _subscription);
  };

  class EventInterpreterStateSensorPlugin : public EventInterpreterPlugin {
    private:
      __DECL_LOG_CHANNEL__
    public:
      EventInterpreterStateSensorPlugin(EventInterpreter* _pInterpreter);
      virtual ~EventInterpreterStateSensorPlugin() {};
      virtual void subscribe();
      virtual void handleEvent(Event& _event, const EventSubscription& _subscription);
  };

  class EventInterpreterHeatingMonitorPlugin : public EventInterpreterPlugin {
    private:
      __DECL_LOG_CHANNEL__
    public:
      EventInterpreterHeatingMonitorPlugin(EventInterpreter* _pInterpreter);
      virtual ~EventInterpreterHeatingMonitorPlugin() {};
      virtual void subscribe();
      virtual void handleEvent(Event& _event, const EventSubscription& _subscription);
  };

  class EventInterpreterHeatingValveProtectionPlugin : public EventInterpreterPlugin {
    private:
      __DECL_LOG_CHANNEL__
    public:
      EventInterpreterHeatingValveProtectionPlugin(EventInterpreter* _pInterpreter);
      virtual ~EventInterpreterHeatingValveProtectionPlugin() {};
      virtual void subscribe();
      virtual void handleEvent(Event& _event, const EventSubscription& _subscription);
  };

  class EventInterpreterDebugMonitorPlugin : public EventInterpreterPlugin {
    private:
      __DECL_LOG_CHANNEL__
    public:
      EventInterpreterDebugMonitorPlugin(EventInterpreter* _pInterpreter);
      virtual ~EventInterpreterDebugMonitorPlugin() {};
      virtual void subscribe();
      virtual void handleEvent(Event& _event, const EventSubscription& _subscription);
  };

  class EventInterpreterDatabaseUpdatePlugin : public EventInterpreterPlugin {
    public:
      EventInterpreterDatabaseUpdatePlugin(EventInterpreter* _pInterpreter);
      virtual ~EventInterpreterDatabaseUpdatePlugin() {};
      virtual void subscribe();
      virtual void handleEvent(Event& _event, const EventSubscription& _subscription);
  };

  class AutoclusterUpdatePlugin : public EventInterpreterPlugin {
      private:
        __DECL_LOG_CHANNEL__
      public:
      AutoclusterUpdatePlugin(EventInterpreter* _pInterpreter);
      virtual ~AutoclusterUpdatePlugin() {};
      virtual void subscribe();
      virtual void handleEvent(Event& _event, const EventSubscription& _subscription);
  };

  class LockClusterPlugin : public EventInterpreterPlugin {
      private:
        __DECL_LOG_CHANNEL__
      public:
        LockClusterPlugin(EventInterpreter* _pInterpreter);
      virtual ~LockClusterPlugin() {};
      virtual void subscribe();
      virtual void handleEvent(Event& _event, const EventSubscription& _subscription);
  };

  class ClusterWeatherProtectionPlugin : public EventInterpreterPlugin {
      private:
        __DECL_LOG_CHANNEL__
        void handleModelReadyEvent();
        void handleClusterCheckWindEvent(Event& _event);
      public:
      ClusterWeatherProtectionPlugin(EventInterpreter* _pInterpreter);
      virtual ~ClusterWeatherProtectionPlugin() {};
      virtual void subscribe();
      virtual void handleEvent(Event& _event, const EventSubscription& _subscription);
  };

  class UpdateShadeStopTimingsPlugin : public EventInterpreterPlugin {
    public:
      UpdateShadeStopTimingsPlugin(EventInterpreter* _pInterpreter);
      virtual ~UpdateShadeStopTimingsPlugin() {};
      virtual void subscribe();
      virtual void handleEvent(Event& _event, const EventSubscription& _subscription);
  };

  class UpdateButtonUsagePlugin : public EventInterpreterPlugin {
    public:
      UpdateButtonUsagePlugin(EventInterpreter* _pInterpreter);
      virtual ~UpdateButtonUsagePlugin() {};
      virtual void subscribe();
      virtual void handleEvent(Event& _event, const EventSubscription& _subscription);
  };

  class SunPositionSensorPlugin : public EventInterpreterPlugin {
    public:
      SunPositionSensorPlugin(EventInterpreter* _pInterpreter);
      virtual ~SunPositionSensorPlugin() {};
      virtual void subscribe();
      virtual void handleEvent(Event& _event, const EventSubscription& _subscription);
  };

} // namespace dss

#endif /* EVENTINTERPRETERPLUGINS_H_ */
