/*
    Copyright (c) 2021 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/
#pragma once

#include <trompeloeil.hpp>

#include "metering/archives-fwd.h"
#include "metering/metering.h"
#include "metering/power-states.h"
#include "metering/readings/open-metering-system.h"
#include "metering/readings/modbusmeteringsystem.h"

namespace dss::metering {

class MeteringMock : public trompeloeil::mock_interface<Metering> {
public:
  IMPLEMENT_CONST_MOCK0(getSerialPortConfig);
  IMPLEMENT_MOCK1(setSerialPortConfig);
  IMPLEMENT_CONST_MOCK0(getModbusAttachedMeters);
  IMPLEMENT_MOCK1(addModbusAttachedMeter);
  IMPLEMENT_MOCK1(deleteModbusAttachedMeter);
  IMPLEMENT_CONST_MOCK0(getModbusSupportedMeters);
  IMPLEMENT_CONST_MOCK0(getModbusUserMeters);
  IMPLEMENT_MOCK1(addModbusSupportedMeter);
  IMPLEMENT_MOCK1(deleteModbusSupportedMeter);
  IMPLEMENT_CONST_MOCK0(openMeteringSystemOrigin);
  IMPLEMENT_CONST_MOCK0(meterings);
  IMPLEMENT_CONST_MOCK0(getConfig);
  IMPLEMENT_CONST_MOCK1(getEnergy);
  IMPLEMENT_CONST_MOCK1(getConsumedPower);
  IMPLEMENT_CONST_MOCK1(getProducedPower);
  IMPLEMENT_CONST_MOCK0(getConsumedPowerApartment);
  IMPLEMENT_CONST_MOCK0(getProducedPowerApartment);
  IMPLEMENT_CONST_MOCK1(getLastEnergy);
  IMPLEMENT_CONST_MOCK1(getProducedEnergy);
  IMPLEMENT_CONST_MOCK3(getLastReading);
  IMPLEMENT_CONST_MOCK5(getSeriesValue);
  IMPLEMENT_MOCK2(createState);
  IMPLEMENT_MOCK2(removeState);
  IMPLEMENT_MOCK2(triggerState);
  IMPLEMENT_MOCK1(setMeteringBusInterface);
  IMPLEMENT_CONST_MOCK0(validateAndConvertArchives);
  MAKE_MOCK1(addReading, void(readings::OpenMeteringSystem&& p1Reading));
  MAKE_MOCK1(addReading, void(readings::ModbusMeteringSystem&& modbusReading));
  IMPLEMENT_MOCK2(addConsumedEnergy);
  IMPLEMENT_MOCK2(addProducedEnergy);
  MAKE_MOCK2(get15minConsumed, double(std::vector<Dsuid>, SeriesTypes));
  MAKE_MOCK2(get15minProduced, double(std::vector<Dsuid>, SeriesTypes));
  MAKE_MOCK2(get15minConsumed, double(std::vector<std::string>, SeriesTypes));
  MAKE_MOCK2(get15minProduced, double(std::vector<std::string>, SeriesTypes));
  IMPLEMENT_MOCK1(get15minConsumedOMS);
  IMPLEMENT_MOCK1(get15minProducedOMS);
  IMPLEMENT_MOCK6(getSeries);
  IMPLEMENT_MOCK6(getProducedSeries);
  IMPLEMENT_MOCK7(getModbusSeries);
  IMPLEMENT_CONST_MOCK1(flushDb);

  // integration tests (dsstests)
  IMPLEMENT_CONST_MOCK0(getArchivesStorage);
  IMPLEMENT_CONST_MOCK0(getArchivesSanity);
};

} // namespace dss::metering
