#include "socket-writer.h"

static ds::LogChannel dsLogChannel("modbusreaderSocketWriter", ds::LogSeverity::DEBUG);

namespace dss::modbusreader {

void SocketClient::send(std::string&& message) {
  m_buffer = std::move(message);
  doConnect();
}

void SocketClient::doConnect() {
  DS_DEBUG("Connect");
  m_socket.async_connect(m_endpoint, [self = shared_from_this()](boost::system::error_code ec) {
    std::cerr << "=================================" << std::endl;
    if (ec) {
      DS_WARNING("Could not connect", ec, self->m_endpoint.path());
      return;
    }
    self->doWrite();
  });
}

void SocketClient::disconnect() {
  // m_socket.close(boost::system::error_code));
}

void SocketClient::doWrite() {
  DS_NOTICE("Writing ", m_buffer);

  m_socket.async_write_some(boost::asio::buffer(m_buffer),
                            [self = shared_from_this()](boost::system::error_code ec, std::size_t size) {
                              DS_NOTICE("TRANSFERRED ", size, self->m_buffer.size());
                              if (ec) {
                                DS_ERROR(ec);
                                return;
                              }
                              // don't schedule another task should end the client
                            });
}

} // namespace dss::modbusreader
