/*
    Copyright (c) 2018 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/

#include "actionrequestexecutor.h"

#include <ds/catch/catch.h>
#include <ds/log.h>

#include "apartment-mock.h"
#include "businterface-mock.h"
#include "device-mock.h"
#include "sceneaccess.h"

static ds::LogChannel dsLogChannel("modelActionRequestCheckerTest");
static const char* TAGS = "[actionrequestcheckertest]";

using namespace dss;
using namespace dss::model;
using namespace std::literals;
using namespace trompeloeil;
using trompeloeil::_;

namespace {
SCENARIO("ActionRequestChecker", TAGS) {
  ActionRequestInterfaceMock actionRequestInterface{};
  ApartmentMock apartment{};

  ActionRequestExecutor actionRequestWrapper(apartment, actionRequestInterface);

  SECTION("device") {
    auto target_ = createDevice(apartment, {.hardwareType = HardwareType::GR_KL300});
    auto& target = *target_;

    WHEN("manual call") {
      Caller caller{.category = SceneAccessCategory::manual};

      THEN("calls allowed") {
        uint8_t value{0x7};
        auto scene = Scene2;
        bool force{false};

        trompeloeil::sequence seq;

        REQUIRE_CALL(actionRequestInterface, callScene(&target, SceneInc, Caller(caller), false)).IN_SEQUENCE(seq);
        actionRequestWrapper.increaseValue(target, Caller(caller));

        REQUIRE_CALL(actionRequestInterface, callScene(&target, SceneDec, Caller(caller), false)).IN_SEQUENCE(seq);
        actionRequestWrapper.decreaseValue(target, Caller(caller));

        REQUIRE_CALL(actionRequestInterface, setValue(&target, value, Caller(caller))).IN_SEQUENCE(seq);
        actionRequestWrapper.setValue(target, Caller(caller), value);

        REQUIRE_CALL(actionRequestInterface, callScene(&target, scene, Caller(caller), false)).IN_SEQUENCE(seq);
        actionRequestWrapper.callScene(target, Caller(caller), scene, force);

        REQUIRE_CALL(actionRequestInterface, callSceneMin(&target, scene, Caller(caller))).IN_SEQUENCE(seq);
        actionRequestWrapper.callSceneMin(target, Caller(caller), scene);

        REQUIRE_CALL(actionRequestInterface, saveScene(&target, scene, Caller(caller))).IN_SEQUENCE(seq);
        actionRequestWrapper.saveScene(target, Caller(caller), scene);

        REQUIRE_CALL(actionRequestInterface, undoScene(&target, scene, Caller(caller))).IN_SEQUENCE(seq);
        actionRequestWrapper.undoScene(target, Caller(caller), scene);

        REQUIRE_CALL(actionRequestInterface, undoSceneLast(&target, Caller(caller))).IN_SEQUENCE(seq);
        actionRequestWrapper.undoSceneLast(target, Caller(caller));

        REQUIRE_CALL(actionRequestInterface, blink(&target, Caller(caller))).IN_SEQUENCE(seq);
        actionRequestWrapper.blink(target, Caller(caller));

        REQUIRE_CALL(actionRequestInterface,
                     increaseOutputChannelValue(&target, ChannelType::brightness, Caller(caller)))
            .IN_SEQUENCE(seq);
        actionRequestWrapper.increaseOutputChannelValue(target, Caller(caller), ChannelType::brightness);
        REQUIRE_CALL(actionRequestInterface,
                     decreaseOutputChannelValue(&target, ChannelType::brightness, Caller(caller)))
            .IN_SEQUENCE(seq);
        actionRequestWrapper.decreaseOutputChannelValue(target, Caller(caller), ChannelType::brightness);
        REQUIRE_CALL(actionRequestInterface, stopOutputChannelValue(&target, ChannelType::brightness, Caller(caller)))
            .IN_SEQUENCE(seq);
        actionRequestWrapper.stopOutputChannelValue(target, Caller(caller), ChannelType::brightness);

        auto sensorValue = 0.7;
        REQUIRE_CALL(actionRequestInterface,
                     pushSensor(&target, Dsuid{}, SensorType::ElectricMeter, sensorValue, Caller(caller)))
            .IN_SEQUENCE(seq);
        actionRequestWrapper.pushSensor(target, Caller(caller), Dsuid{}, SensorType::ElectricMeter, sensorValue);
      }
    }
  }
}
} // namespace
