/*
    Copyright (c) 2015 digitalSTROM.org, Zurich, Switzerland

    Author: Remy Mahler, digitalSTROM AG <remy.mahler@digitalstrom.com>

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

*/

#pragma once

#include <boost/noncopyable.hpp>
#include <boost/thread/mutex.hpp>

#include "data_types.h"
#include "model/device.h"

namespace dss {

class Apartment;
class Device;
class Cluster;

class AutoClusterMaintenance : public boost::noncopyable {

public:
  AutoClusterMaintenance(Apartment *_apartment);
  ~AutoClusterMaintenance();

  void globalConsistencyCheck();

  void consistencyCheck(Device &_device);
  void joinIdenticalClusters();
  void cleanupEmptyCluster();

protected:
  std::shared_ptr<Cluster> findOrCreateCluster(CardinalDirection_t _cardinalDirection,
                                               WindProtectionClass_t _protection); // protected for unit tests

private:
  void removeDeviceFromCluster(Device &_device, std::shared_ptr<Cluster> _cluster);
  void assignDeviceToCluster(Device &_device, std::shared_ptr<Cluster> _cluster);

  void moveClusterDevices(std::shared_ptr<Cluster> _clusterSource, std::shared_ptr<Cluster> _clusterDestination);
  void removeInvalidAssignments(Device &_device);
  std::vector<std::shared_ptr<Cluster> > getUnlockedClusterAssignment(Device &_device);
  int getFirstLockedClusterAssignment(Device &_device);
  void removeEmptyAutomaticCluster();

  void busUpdateCluster(std::shared_ptr<Cluster> _cluster);

private:
  Apartment *m_pApartment;
};

} // namespace dss
