/*
    Copyright (c) 2010 digitalSTROM.org, Zurich, Switzerland

    Author: Patrick Staehlin, futureLAB AG <pstaehlin@futurelab.ch>

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <thread>

#include "deviceinterface.h"
#include "model/modelconst.h"

namespace dss {
//================================================== IDeviceInterface

void IDeviceInterface::turnOn(const CallOrigin _origin, const SceneAccessCategory _category) {
  callScene(_origin, _category, SceneMax, "");
}

void IDeviceInterface::turnOff(const CallOrigin _origin, const SceneAccessCategory _category) {
  callScene(_origin, _category, SceneMin, "");
}

void IDeviceInterface::async(std::function<void()>&& call, AsyncHandler&& handler) {
  std::thread([call, handler]() {
    try {
      call();
      handler({});
    } catch (const std::exception& e) {
      handler(e);
    }
  }).detach();
}
} // namespace dss
