/*
    Copyright (c) 2022 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#include <string_view>
#include <vector>

namespace dss {

std::vector<std::string_view> splitBuffer(std::string_view buffer);

std::vector<std::pair<int, int>> shortestPath(const std::vector<std::string_view>& left,
                                              const std::vector<std::string_view>& right);

std::vector<std::string> createPatch(const std::vector<std::string_view>&, const std::vector<std::string_view>&,
                                     const std::vector<std::pair<int, int>>&, int context = -1);

std::vector<std::string> formatHook(const std::vector<std::string_view>&, const std::vector<std::string_view>&,
                                    const std::vector<std::pair<int, int>>&,
                                    std::vector<std::pair<int, int>>::const_iterator&, int context = -1);

class Patch {
public:
  Patch(const std::string& new_, const std::string& old);
  Patch(std::vector<std::string>&& new_, std::vector<std::string>&& old);
  std::vector<std::string> diff(int context = 2) const;

private:
  std::vector<std::string_view> m_new;
  std::vector<std::string_view> m_old;
  std::vector<std::pair<int, int>> editPath;

  std::vector<std::string> m_newOwn; // keep buffer alive until diff is called
  std::vector<std::string> m_oldOwn;
};

} // namespace dss
