/*
    Copyright (c) 2010 digitalSTROM.org, Zurich, Switzerland
    Copyright (c) 2022 Aizo AG, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#include <digitalSTROM/dsm-api-v2/dsm-api.h>

#include "model/device.h"
#include "taskprocessor.h"

namespace dss {

class ConfigReaderHelper {
public:
  ConfigReaderHelper(const std::string& _busConnection);
  virtual ~ConfigReaderHelper();

  virtual void setup(std::shared_ptr<Device> _device);

  uint16_t getDeviceConfigWord(uint8_t _configClass, uint8_t _configIndex) const;
  uint8_t getDeviceConfig(uint8_t _configClass, uint8_t _configIndex) const;

protected:
  std::string m_busConnection;
  DsmApiHandle_t m_dsmApiHandle;
  devid_t m_deviceAdress;
  dsuid_t m_dsmId;
  std::shared_ptr<DSMeter> m_dsm;
  int m_revisionID;
};

class OEMDataReader : public Task, public ConfigReaderHelper, public std::enable_shared_from_this<OEMDataReader> {
public:
  OEMDataReader(const std::string& _busConnection);
  virtual ~OEMDataReader();
  virtual void run();
};

class TNYConfigReader : public Task, public ConfigReaderHelper, public std::enable_shared_from_this<TNYConfigReader> {
public:
  TNYConfigReader(const std::string& _busConnection);
  virtual ~TNYConfigReader();
  virtual void run();
};

} // namespace dss
