@0x912c62db6b590037; # unique file ID, generated by `capnp id`

using import "form.capnp".option;
using import "form.capnp".field;
using import "form.capnp".View;

using Cxx = import "/capnp/c++.capnp";
$Cxx.namespace("dss::view::deviceConfig::dssCapnp");

enum Application {
    none @0;
    lights @1;
    blinds @2;
    heating @3;
    audio @4;
    video @5;
    security @14;
    access @15;
    joker @6;
    cooling @7;
    ventilation @8;
    window @9;
    recirculation @10;
    temperatureControl @11;
    apartmentVentilation @12;
    awnings @16;
    apartmentRecirculation @13;
}

struct Cluster {
    id @0 :UInt32;
    application @1 :Application;
    title @2 :Text;
}

struct Args {
    memberOfClusters @0 :List(Cluster); # list of clusters the device is member of

    showApplication @1 :Bool;
    showTemperatureControlApplication @2 :Bool;
    showApartmentApplications @3 :Bool;

    showInput @4 :Bool;
    enum InputTypeMode {
        hiddenPushButton @0;
        shownPushButtonAndSensor @1;
    }
    inputTypeMode @5 :InputTypeMode;

    showPushButtonApplication @6 :Bool;
    showPushButtonDeviceMode @7 :Bool;
    showPushButtonDisabledMode @8 :Bool;

    showPushButtonMode @9 :Bool;

    showOutput @10 :Bool;
    showOutputAttachedDevice @11 :Bool;
}

struct Value
{
    enum InputType {
        pushButton @0;
        state @1 $option(
            showIf = "args.inputTypeMode == 'shownPushButtonAndSensor'");
    }
    inputType @0 :InputType $field(
        title = "Type",
        showIf = "args.showInput && args.inputTypeMode == 'shownPushButtonAndSensor'",
        watches = [
            (
                watch =  "args.inputTypeMode == 'hiddenPushButton'",
                do = "if (newWatchValue) { value.inputType = 'pushButton' }")]);

    enum Application {
        lights @0 $option(icon = "colorYellow"); # 1
        blinds @1 $option(icon = "colorGrey", title = "Shade"); # 2
        heating @2 $option(icon = "colorBlue"); #3
        audio @3 $option(icon = "colorCyan"); #4
        video @4 $option(icon = "colorMagenta"); #5
        joker @5 $option(icon = "colorBlack"); #8
        recirculation @6 $option(icon = "colorRoomVentilation"); #12
        security @7 $option(
            icon ="colorRed",
            title = "Reserved",
            showIf = "initValue.application === 'security'"); #6
            # #1891 - keep value but replace with "reserved" entry
        access @8 $option(
            icon ="colorGreen",
            title = "Reserved",
            showIf = "initValue.application === 'access'"); # 7
            # #1891 - keep value but replace with "reserved" entry
        window @9 $option(icon = ""); #11
        apartmentVentilation @10 $option(
            icon ="colorAptVentilation",
            showIf = "args.showApartmentApplications"); #64
        temperatureControl @11 $option(
            icon ="colorTemperatureControl",
            showIf = "args.showTemperatureControlApplication"); #48
        ventilation @12; #10
        awnings @13 $option(icon = "colorGrey", title = "Awnings"); # 47
    }

    application @1 :Application $field(
        title = "Color group",
        iconInOptions = true,
        showIf = "args.showApplication || (args.showInput && args.showPushButtonApplication && value.inputType === 'pushButton')");
        # TODO(someday): Use generic ApplicationType enum type
        # and move annotations attached to Application enumerants to
        # application field annotation options = [(id = "lights", ....), ...]

    enum PushButtonTarget {
        device @0 $option(
            title = "Device push-button",
            showIf = "args.showPushButtonDeviceMode && value.application !== 'joker'");
        zone @1 $option(
            title = "Room push-button",
            showIf = "value.application !== 'joker'");
        area @2 $option(
            title = "Area push-button",
            showIf = "value.application === 'lights' || value.application === 'blinds'");
        cluster @3 $option(
            title = "Group push-button",
            showIf = "args.memberOfClusters && args.memberOfClusters.length > 0 && args.memberOfClusters[0].application === value.application");
        app @4 $option(
            title = "App button",
            showIf = "value.application === 'joker'");
        alarm @5 $option(
            showIf = "value.application === 'joker'");
        panic @6 $option(
            showIf = "value.application === 'joker'");
        leavingHome @7 $option(
            showIf = "value.application === 'joker'");
        doorBell @8 $option(
            showIf = "value.application === 'joker'");
        disabled @9 $option(
            showIf = "args.showPushButtonDisabledMode");
        # sensor; not for UMR200 (enabletypehack === true) - although I (brano) happed to see it in UI
    }
    pushButtonTarget @2 :PushButtonTarget $field(
        title = "Push-button",
        showIf = "args.showInput && value.inputType === 'pushButton'");

    enum PushButtonTargetArea {
        area1 @0;
        area2 @1;
        area3 @2;
        area4 @3;
    }
    pushButtonTargetArea @3 :PushButtonTargetArea $field(
        title = "Area",
        showIf = "args.showInput && value.inputType === 'pushButton' && value.pushButtonTarget === 'area'");

    pushButtonTargetClusterId @4 :UInt32 $field(
        title = "Group",
        optionsQuery = (id = "x.id", title = "x.title", forXIn = "args.memberOfClusters"),
        showIf = "args.showInput && value.inputType === 'pushButton' && value.pushButtonTarget === 'cluster'");
        # TODO(soon): refactor from Text to UInt32

    enum PushButtonTargetPresets {
        devicePresets2To4 @0 $option(
            title = "Presets 2-4",
            showIf = "value.pushButtonTarget === 'device'");
        zonePresets1To4 @1 $option(
            title = "Presets 0-4",
            showIf = "value.pushButtonTarget === 'zone' || value.pushButtonTarget === 'cluster'");
        zonePresets10To14 @2 $option(
            title = "Presets 10-14",
            showIf = "value.pushButtonTarget === 'zone' || value.pushButtonTarget === 'cluster'");
        zonePresets20To24 @3 $option(
            title = "Presets 20-24",
            showIf = "value.pushButtonTarget === 'zone' || value.pushButtonTarget === 'cluster'");
        zonePresets30To34 @4 $option(
            title = "Presets 30-34",
            showIf = "value.pushButtonTarget === 'zone' || value.pushButtonTarget === 'cluster'");
        zonePresets40To44 @5 $option(
            title = "Presets 40-44",
            showIf = "value.pushButtonTarget === 'zone' || value.pushButtonTarget === 'cluster'");
        area1Presets1To4 @6 $option(
            title = "Presets 2-4",
            showIf = "value.pushButtonTarget === 'area' && value.pushButtonTargetArea === 'area1'");
        area1Presets12To14 @10 $option(
            title = "Presets 12-14",
            showIf = "value.pushButtonTarget === 'area' && value.pushButtonTargetArea === 'area1'");
        area2Presets1To4 @7 $option(
            title = "Presets 2-4",
            showIf = "value.pushButtonTarget === 'area' && value.pushButtonTargetArea === 'area2'");
        area2Presets22To24 @11 $option(
            title = "Presets 22-24",
            showIf = "value.pushButtonTarget === 'area' && value.pushButtonTargetArea === 'area2'");
        area3Presets1To4 @8 $option(
            title = "Presets 2-4",
            showIf = "value.pushButtonTarget === 'area' && value.pushButtonTargetArea === 'area3'");
        area3Presets32To34 @12 $option(
            title = "Presets 32-34",
            showIf = "value.pushButtonTarget === 'area' && value.pushButtonTargetArea === 'area3'");
        area4Presets1To4 @9 $option(
            title = "Presets 2-4",
            showIf = "value.pushButtonTarget === 'area' && value.pushButtonTargetArea === 'area4'");
        area4Presets42To44 @13 $option(
            title = "Presets 42-44",
            showIf = "value.pushButtonTarget === 'area' && value.pushButtonTargetArea === 'area4'");
    }
        # TODO(naming): area4Presets42To44 vs area4Presets4

    pushButtonTargetPresets @5 :PushButtonTargetPresets $field(
        showIf = "args.showInput && value.inputType === 'pushButton'",
        enableIf = "value.pushButtonTarget === 'device' || value.pushButtonTarget === 'zone' || value.pushButtonTarget === 'area' || value.pushButtonTarget === 'cluster'");

    pushButtonLocalPriorityFor1xClick @6 :Bool $field(
        title = "Local priority for 1x click",
        showIf = "args.showInput && value.inputType === 'pushButton'",
        enableIf = "value.pushButtonTarget === 'area'",
        watches = [
            (
                watch =  "value.application === 'lights'",
                do = "if (newWatchValue) { value.pushButtonLocalPriorityFor1xClick = true; }",
                doOnInit = false),
            (
                watch =  "value.application === 'blinds'",
                do = "if (newWatchValue) { value.pushButtonLocalPriorityFor1xClick = false; }",
                doOnInit = false)]);

    pushButtonCallsPresent @7 :Bool = true $field(
        title = "Calls present",
        showIf = "args.showInput && value.inputType === 'pushButton'");

    enum PushButtonMode {
        mode1way @0  $option(
            title = "1-way");
        mode2wayDown @1 $option(
            title = "{{value.application === 'blinds' ? _('2-way down') : _('2-way off')}}",
            showIf = "args.showPushButtonMode && value.application !== 'joker'");
        mode2wayUp @2 $option(
            title = "{{value.application === 'blinds' ? _('2-way up') : _('2-way on')}}",
            showIf = "args.showPushButtonMode && value.application !== 'joker'");
    }
    # TODO(naming): mode2wayDown, mode2wayUp or mode2wayOff, mode2wayOn?
    # off, on for most applications in UI, up,down for blinds in UI
    # 2-Wege-Taster-Ab, 2-Wege-Taster-Auf in https://intranet.aizo.net/Wiki/Parametersatz.aspx
    # 2way_down. 2way_up in json/device/setButtonInputMode
    # TWO_WAY_DW, TWO_WAY_UP in modelconst.h
    pushButtonMode @8 :PushButtonMode $field(
        title = "Button function",
        showIf = "args.showInput && args.showPushButtonMode && value.inputType === 'pushButton' && value.application !== 'joker'");

    enum StateType {
        generic @0 $option(title = "App Mode");
        presence @1;
        twilight @2;
        motion @3 $option(title = "Motion detector");
        smoke @4 $option(title = "Smoke detector");
        windStrengthAboveLimit @5 $option(title = "Wind monitor");
        rain @6 $option(title = "Rain monitor");
        sunRadiation @7 $option(title = "Solar radiation");
        sunProtection @8 $option(title = "Sun protection");
        frost @9 $option(title = "Frost monitor");
        temperatureBelowLimit @10 $option(title = "Thermostat");
        windowIsOpen @11 $option(title = "Window contact");
        doorIsOpen @12 $option(title = "Door contant");
        garageDoorIsOpen @13 $option(title = "Garage door contact");
        heatingSystemEnabled @14 $option(title = "Heating operation on/off");
        changeOverSignal @15 $option(title = "Change-over heating/cooling");
        malfunction @16;
        service @17;
    }
    stateType @9 :StateType $field(
        title = "Sensor function",
        showIf = "args.showInput && value.inputType === 'state'");

    enum StateTarget {
        wholeApartment @0;
        apartmentVentilation @1 $option(
            showIf = "args.showApartmentApplications && (value.stateType === 'malfunction' || value.stateType === 'service')");
        ventilation @2;
        recirculation @3;
    }
    stateTarget @10 :StateTarget $field(
        showIf = "args.showInput && value.inputType === 'state' && (value.stateType === 'malfunction' || value.stateType === 'service')");

    enum StateMode {
        standard @0;
        inverted @1;
        risingEdge1 @2 $option(title = "Rising edge on");
        fallingEdge1 @3 $option(title = "Failing edge on");
        risingEdge0 @4 $option(title = "Rising edge off");
        fallingEdge0 @5 $option(title = "Failing edge off");
        toggleModeOnRisingEdge @6 $option(title = "Rising edge on/off");
        toggleModeOnFallingEdge @7 $option(title = "Rising edge on/off");
    }
    stateMode @11 :StateMode $field(
        title = "Input",
        showIf = "args.showInput && value.inputType === 'state'");

}

const umr200InputArgs : Args = (
    memberOfClusters = [
        (id=21, application = lights, title="1st floor lights"),
        (id=23, application = lights, title="2nd floor lights")],
    showApartmentApplications = true, showInput = true, inputTypeMode = shownPushButtonAndSensor, showPushButtonApplication = true, showPushButtonMode = true);

const umr200OutputArgs : Args = (
    showApplication = true,
    showApartmentApplications = true,
    showOutput = true);

const args : Args = .umr200InputArgs;
const value : Value = ();
const view : View = (
    tabs = [
        (title = "Settings", rows = [
            (item = (fieldId = "inputType")),
            (item = (fieldId = "application")),
            (item = (fieldId = "pushButtonTarget"), subItems = [
                (fieldId = "pushButtonTargetArea"),
                (fieldId = "pushButtonTargetClusterId")]),
            (item = (fieldId = "pushButtonMode")),
            (item = (fieldId = "stateType"), subItems = [
                (fieldId = "stateTarget")]),
            (item = (fieldId = "stateMode"))]),
        (title = "Advanced Settings", rows = [
            (item = (fieldId = "pushButtonTargetPresets")),
            (item = (fieldId = "pushButtonLocalPriorityFor1xClick")),
            (item = (fieldId = "pushButtonCallsPresent"))])]);


struct Data {
    args @0 :Args;
    value @1 :Value;
    # TODO(someday): view
}
