/*
    Copyright (c) 2010 digitalSTROM.org, Zurich, Switzerland

    Author: Patrick Staehlin, futureLAB AG <pstaehlin@futurelab.ch>

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef ZONEREQUESTHANDLER_H_
#define ZONEREQUESTHANDLER_H_

#include "dssfwd.h"
#include "web/webrequests.h"

namespace dss {

  class Apartment;
  class StructureModifyingBusInterface;
  class StructureQueryBusInterface;
  class Zone;

  class ZoneRequestHandler : public WebServerRequestHandlerJSON {
  public:
    ZoneRequestHandler(Apartment& _apartment);
    virtual WebServerResponse jsonHandleRequest(const RestfulRequest& _request, std::shared_ptr<Session> _session, const struct mg_connection* _connection);

    // methods to render different heating info in JSON format
    static void addTemperatureControlStatus(JSONWriter& json, std::shared_ptr<Zone> pZone);
    static void addTemperatureControlConfig(JSONWriter& json, std::shared_ptr<Zone> pZone);
    static void addTemperatureControlConfig2(JSONWriter& json, std::shared_ptr<Zone> pZone);
    static void addTemperatureControlValues(JSONWriter& json, std::shared_ptr<Zone> pZone);
  private:
    Apartment& m_Apartment;

    std::string getLastCalledScene(std::shared_ptr<Zone> pZone, std::shared_ptr<Group> pGroup, const RestfulRequest& _request);
    std::string getName(std::shared_ptr<Zone> pZone, std::shared_ptr<Group> pGroup, const RestfulRequest& _request);
    std::string setName(std::shared_ptr<Zone> pZone, std::shared_ptr<Group> pGroup, const RestfulRequest& _request);
    std::string sceneSetName(Group&, const RestfulRequest&);
    std::string sceneGetName(std::shared_ptr<Zone> pZone, std::shared_ptr<Group> pGroup, const RestfulRequest& _request);
    std::string getReachableScenes(std::shared_ptr<Zone> pZone, std::shared_ptr<Group> pGroup, const RestfulRequest& _request);
    std::string getTemperatureControlStatus(std::shared_ptr<Zone> pZone, std::shared_ptr<Group> pGroup, const RestfulRequest& _request);
    std::string getTemperatureControlConfig(std::shared_ptr<Zone> pZone, std::shared_ptr<Group> pGroup, const RestfulRequest& _request);
    std::string getTemperatureControlConfig2(std::shared_ptr<Zone> pZone, std::shared_ptr<Group> pGroup, const RestfulRequest& _request);
    std::string setTemperatureControlConfig(std::shared_ptr<Zone> pZone, std::shared_ptr<Group> pGroup, const RestfulRequest& _request);
    std::string setTemperatureControlConfig2(std::shared_ptr<Zone> pZone, std::shared_ptr<Group> pGroup, const RestfulRequest& _request);
    std::string getTemperatureControlValues(std::shared_ptr<Zone> pZone, std::shared_ptr<Group> pGroup, const RestfulRequest& _request);
    std::string setTemperatureControlValues(std::shared_ptr<Zone> pZone, std::shared_ptr<Group> pGroup, const RestfulRequest& _request);
    std::string getTemperatureControlInternals(std::shared_ptr<Zone> pZone, std::shared_ptr<Group> pGroup, const RestfulRequest& _request);
    std::string setStatusField(std::shared_ptr<Zone> pZone, std::shared_ptr<Group> pGroup, const RestfulRequest& request);
    std::string setSensorSource(std::shared_ptr<Zone> pZone, std::shared_ptr<Group> pGroup, const RestfulRequest& _request);
    std::string clearSensorSource(std::shared_ptr<Zone> pZone, std::shared_ptr<Group> pGroup, const RestfulRequest& _request);
    std::string getAssignedSensors(std::shared_ptr<Zone> pZone, std::shared_ptr<Group> pGroup, const RestfulRequest& _request);
    std::string getSensorValues(std::shared_ptr<Zone> pZone, std::shared_ptr<Group> pGroup, const RestfulRequest& _request);
    std::string getActions(std::shared_ptr<Zone> pZone, std::shared_ptr<Group> pGroup, const RestfulRequest& _request);
    std::string callAction(std::shared_ptr<Zone> pZone, std::shared_ptr<Group> pGroup, const RestfulRequest& _request);

  }; // ZoneRequestHandler
}

#endif /* ZONEREQUESTHANDLER_H_ */
