/*
    Copyright (c) 2017 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#include <iosfwd>
#include <string>

#include "ds/json.h"
#include "ds/reflect.h"

DS_REFLECTED_ENUM(
    (dss)(websocket), MessageType,
    (request) // Calls `method` with `params` at `url` identified by `id`. The call is answered with reply once.
    (event)   // Sends `event` to unreplied request `id`.
    (reply)   // Replies to request `id` with success (missing `error'`field) or failure (present `error` field)
    (cancel)  // Cancels request `id`.
);

namespace dss {

namespace websocket {

struct MessageError {
  /// Must not be empty if an error happens. Empty message means no error.
  std::string message;
};

struct Message {
  MessageType type{};
  std::string id;
  std::string url;         // When type=='request'. Like url path.
  std::string method;      // When type=='request'. Method to be invoked on url. May be empty for default method.
  ds::JsonFragment params; // When type=='request'. Request arguments. Type is specific for url and method. Must be
                           // missing of json struct.
  ds::JsonFragment event;  // When type=='event'. // any json object. Type is specific for url and method. Must be
                           // missing of json struct.
  ds::JsonFragment
      result; // When type=='reply'. Request reply. Type is specific for url and method. Must be missing of json struct.
  MessageError error{}; // When type=='reply'. // any json object. Type is specific for url and method.
};

} // namespace websocket

} // namespace dss

DS_REFLECT_STRUCT((dss)(websocket), MessageError, (message));
DS_REFLECT_STRUCT((dss)(websocket), Message, (type)(id)(url)(method)(params)(event)(result)(error));
